/*
Board game rules:
   1. Obtain boardSize from command line or from user.
	2. Designate two spaces for theGoal and thePit.
	3. while (not gameOver)
	4.   
	       Player[i] rolls dice and has new position.
	        if(Goal or Pit)
			      gameOver is true
					announce winner.
			  else
			      describe and record new position of Player[i]
*/
#include <QTextStream>
#include <QCoreApplication>
#include <QStringList>
#include <cstdlib>

QTextStream cout(stdout);
QTextStream cin(stdin);
 
int diceRoll() {
   static const int FACES(6);
   int die1, die2;
   die1 = 1 + rand() % FACES;
   die2 = 1 + rand() % FACES;
   return die1 + die2;
}

int pitOrGoal(int spaces) {
   int res;
   while (1) {
      res = rand() % (spaces);
      if(res > 1)
         return res;
   }
}

int getSpaces(int argc, QCoreApplication& app) {
   const int MINSPACES(5);
   int spaces(0);
   QString ans;
   if(argc > 1) {
      QStringList arglst = app.arguments();
      spaces = arglst[1].toInt();
   }
   if(spaces < 5) {
      while(1) {
         cout << "How many spaces are on the game board? " << flush;
         ans = cin.readLine();
         spaces = ans.toInt();
         if(spaces > MINSPACES) 
            return spaces;
         else
            cout << "At least 5 spaces please ..." << endl;
      }
   }
   return spaces;
}

int currentGame(int spaces, int pit, int goal, bool continuous) {
   int pos0(0), pos1(0);  //Both players start on first space.
   QString ans;
   int roll;
   while(1) {
      roll = diceRoll();
      cout << "You throw " << roll << '\t' << flush;
      if(not continuous){
         ans = cin.readLine();
      }
      pos1 = (pos1 + roll) % spaces;
      if(pos1 == pit) {
         cout << "You fell into the pit - you lose!" << endl;
         return 0;
      }
      if(pos1 == goal) {
         cout << "You reached the goal! You win this one!" << endl;
         return 1;
      }
      cout << "You are now on space " << pos1 << endl;
      roll = diceRoll();
      cout << "I throw " << roll << '\t' << flush;
      if(not continuous){
         ans = cin.readLine();
      }
      pos0 = (pos0 + roll)% spaces;
      if(pos0 == pit){
         cout << "Arrgh! I'm in the pit - you win this one!" << endl;
         return 1; 
      }
      if(pos0 == goal) {
         cout << "I reached the goal!  I win this one!" << endl;
         return 0;
      }
      cout << "I am now on space " << pos0 << endl;
   }
}


int main(int argc, char** argv) {
   QCoreApplication app(argc, argv);
   int spaces(getSpaces(argc, app));
   srand(time(0));   //Set the seed for the rand() function.
   int pit(pitOrGoal(spaces));
   int goal(pitOrGoal(spaces));
   while(pit == goal)  // pit and goal must be separate spaces
      goal = pitOrGoal(spaces);
   cout << "The goal is on space " << goal << " and the pit is on space " << pit
        << endl;
   QString ans;
   cout << "Do you want continuous play (y/N)? " << flush; //No is default.
   ans = cin.readLine();
   bool continuous = (ans.toLower().startsWith('y'));
   // Computer is player0, human is player1
   int wins0(0), wins1(0), who;
   do {
      who = currentGame(spaces, pit, goal, continuous);
      if(who == 0)
         ++wins0;
      else
         ++wins1;
      cout << "So far: You have won " << wins1 << " games, I have won "
           << wins0 << " games.\n" << "Play another (y/N)? " << flush;
      ans = cin.readLine();
   } while(ans.toLower().startsWith('y'));
   cout << "Thanks for playing with me ... I get kinda lonely sometimes."
        << endl;
}

